/*
 * Nomad Instrument Control Software
 *
 * Copyright 2011 Institut Laue-Langevin
 *
 * Licensed under the EUPL, Version 1.1 only (the "License");
 * You may not use this work except in compliance with the Licence.
 * You may obtain a copy of the Licence at:
 *
 * http://joinup.ec.europa.eu/software/page/eupl
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the Licence is distributed on an "AS IS" basis,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the Licence for the specific language governing permissions and
 * limitations under the Licence.
 */

#ifndef COUNTWITHSPECTRO_H_
#define COUNTWITHSPECTRO_H_

#include <Controller.h>
#include "controllers/spectrometer/QE65000Controller.h"
#include "DualSwitchSpectroController.h"
#include "controllers/common/acquisition/TimeCount.h"

namespace d22special {

using namespace spectrometer;

class CountWithSpectro: public ExperimentController, public controller::Stoppable {

public:

	static const std::string TYPE;
	static const int32 XSIZE;
	static const int32 YSIZE;
	static const int32 USB_DELAY_MILLISEC;

	CountWithSpectro(const std::string & name);
	virtual ~CountWithSpectro();

	virtual void postConfiguration();

	virtual void start();
	virtual void stop();

	ControllerPtr<QE65000Controller> spectro;
	ControllerPtr<DualSwitchSpectroController> spectroSwitcher;
	ControllerPtr<acquisition::TimeCount> count;

	Property<int32> integrationTime;
	Property<int32> numSpectroMeasures;
	Property<int32> scansToAverage;
	Property<int32> numTotalMeasures;
	Property<int32> actualPosition; // Property related to D22SampleSequencer
	Property<bool> useSwitcher;
	Property<bool> inSequencerMode; // When being in sequencer mode just shift the data two positions
	Property<bool> useSequencer;

	Property<std::string> timeType;
	Property<float64> countTime;
	Property<std::string> subtitle;

	Property<int32> xSize;
	Property<int32> ySize;
	Property<int32> zSize;

	Property<int32> slotNumber;

	ArrayProperty<float64> yDataArray;
	ArrayProperty<float64> yDataArraySecondChannel;

	DynamicProperty<int32> measureDuration;
	Property<std::string> initialTime;

	void refreshNumSpectroMeasures(int32 number);
	void refreshUseSwitcher(bool value);
	void updateProgression();
	void updateSubtitle();
	void update();
	virtual void setEnabledForLoading(bool all, bool postconf = true);
	virtual void setDisabled(bool all = false);
	virtual void cleanArrays();

private:
	void saveSpectroData();
	void saveSwitcherSpectroData();
	void saveSpectroDataInShiftedMode(); // When CountWithSpectro is used from SampleSequencer
	void startCount();
	void startSpectro();
	void startSwitcher();
	void initArrays();
	void setupCountParams();
	float64 getCountTimeInSec();
	float64 calculateWaitingTimeInSec();

	float64* m_yDataArray;
	float64* m_yDataArraySecondChannel;

	float64* m_yDataOddPositionArray;
	float64* m_yDataEvenPositionArray;
	float64* m_yDataArrayToSave;


};

}

#endif /* COUNTWITHSPECTRO_H_ */
