/*
 * Nomad Instrument Control Software
 *
 * Copyright 2011 Institut Laue-Langevin
 *
 * Licensed under the EUPL, Version 1.1 only (the "License");
 * You may not use this work except in compliance with the Licence.
 * You may obtain a copy of the Licence at:
 *
 * http://joinup.ec.europa.eu/software/page/eupl
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the Licence is distributed on an "AS IS" basis,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the Licence for the specific language governing permissions and
 * limitations under the Licence.
 */

#ifndef MANTID_REDUCTIONCONTROLLERTEST_H
#define MANTID_REDUCTIONCONTROLLERTEST_H

#include <Controller.h>
#include <cameo/cameo.h>

namespace mantid {

class ReductionControllerTest : public ExperimentController,
						public controller::Stoppable {

public:
	//! Type of controller
	static const std::string TYPE;

	ReductionControllerTest(const std::string& name);
	ReductionControllerTest(const ReductionControllerTest& controller);
	virtual ~ReductionControllerTest();

	virtual void postConfiguration();

	virtual void start();
	virtual void stop();

	Property<std::string> serverEndpoint;
	Property<bool> initialized;
	Property<std::string> fileDirectory;
	Property<std::string> filename;
	ArrayProperty<float64> x;
	ArrayProperty<float64> data;

private:
	bool initApplication();
	void readFile(const std::string& nexusFileName, std::string& fileContent);
	void writeFile(const std::string& nexusFileName, const std::string& fileContent);
	void readProcessedFile();

	static const std::string REMOTE_APPLICATION;
	static const std::string RESPONDER;

	std::auto_ptr<cameo::Server> m_server;
	std::auto_ptr<cameo::application::Instance> m_remoteApplication;
};

}

#endif
