/*
 * Nomad Instrument Control Software
 *
 * Copyright 2011 Institut Laue-Langevin
 *
 * Licensed under the EUPL, Version 1.1 only (the "License");
 * You may not use this work except in compliance with the Licence.
 * You may obtain a copy of the Licence at:
 *
 * http://joinup.ec.europa.eu/software/page/eupl
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the Licence is distributed on an "AS IS" basis,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the Licence for the specific language governing permissions and
 * limitations under the Licence.
 */

#ifndef CAENCFDCONTROLLER_H
#define CAENCFDCONTROLLER_H

#include "drivers/caen/caen_cfd/CAENCfdDriver.h"
#include "CAENAdcController.h"
//#include "drivers/ioxos/cfd/CfdDriver.h"

namespace npp {

/*!
 * \class CAENV1725Driver
 * \brief Main class for the CAENV1725Driver device driver
 *
 * \par
 * Class for CAENV1725Driver driver. Implements DeviceDriver methods.
 * \par
 * The CAENV1724Driver device manages the global functionality of the CAEN V1725 VME Digitiser.
 */
class CAENCfdController: public CAENAdcController {

public:

	//! Driver type value
	static const std::string TYPE;

	/*!
	 * \brief Constructor
	 * \param[in] name the name of the device driver
	 */
	CAENCfdController(const std::string& name);

	/*!
	 * \brief Destructor
	 */
	virtual ~CAENCfdController();

	/**
	 * \brief Postconfiguration
	 */
	virtual void postConfiguration();

	// Channels properties
	Property<int32> nbBits;
	Property<float64> signalDecayTime;
	Property<float64> trapezoidFlatTop;
	Property<float64> trapezoidRiseTime;
	Property<float64> flatTopDelay;
	Property<int32> triggerSmoothing;
	Property<float64> signalRiseTime;
	Property<int32> triggerThreshold;
	Property<int32> baselineMean;
	Property<int32> peakMean;
	Property<float64> peakHoldOff;
	Property<float64> baselineHoldOff;
	Property<float64> triggerHoldOff;
	Property<int32> triggerWindowSize;
	Property<int32> triggerWindowEnable;
	Property<int32> digitalGain;
	Property<float64> trapezoidGain;
	Property<int32> decimation;
	Property<int32> cfdActive;
	Property<int32> cfdFraction;
	Property<int32> cfdDelay;
	Property<int32> cfdSignalZero;
	Property<int32> dynamicRange;
	Property<int32> cfdTrap;

	// Channels properties outside dppParam
	Property<int32> useFirstDerivative;

protected:

	DriverPtr<caen_cfd::CAENCfdDriver> m_dgtzDriver;

private:


	virtual void refreshNbChannelsProperty(int32 aValue) throw (CannotSetValue);
	virtual void refreshDisplayChannelProperty(int32 aValue) throw (CannotSetValue);
	virtual void refreshRecordLengthProperty(int32 aValue) throw (CannotSetValue);
	virtual void refreshcfdTrapProperty(int32 aValue) throw (CannotSetValue);
	virtual void refreshDppAcqModeProperty(int32 aValue) throw (CannotSetValue);

	virtual void refreshChannelActiveProperty(int32 aValue) throw (CannotSetValue);

	void refreshSignalDecayTime(float64 aValue) throw (CannotSetValue);
	void refreshTrapezoidFlatTopProperty(float64 aValue) throw (CannotSetValue);
	void refreshTrapezoidRiseTimeProperty(float64 aValue) throw (CannotSetValue);
	void refreshFlatTopDelayProperty(float64 aValue) throw (CannotSetValue);
	void refreshTriggerSmoothing(int32 aValue) throw (CannotSetValue);
	void refreshSignalRiseTimeProperty(float64 aValue) throw (CannotSetValue);
	void refreshTriggerThresholdProperty(int32 aValue) throw (CannotSetValue);
	void refreshBaselineMean(int32 aValue) throw (CannotSetValue);
	void refreshPeakMean(int32 aValue) throw (CannotSetValue);
	void refreshPeaklineHoldOffProperty(float64 aValue) throw (CannotSetValue);
	void refreshBaselineHoldOffProperty(float64 aValue) throw (CannotSetValue);
	void refreshTriggerHoldOffProperty(float64 aValue) throw (CannotSetValue);
	void refreshTriggerWindowSizeProperty(int32 aValue) throw (CannotSetValue);
	void refreshTriggerWindowEnableProperty(int32 aValue) throw (CannotSetValue);
	void refreshDigitalGainProperty(int32 aValue) throw (CannotSetValue);
	void refreshTrapezoidGainProperty(float64 aValue) throw (CannotSetValue);
	void refreshDecimationProperty(int32 aValue) throw (CannotSetValue);
	void refreshDcOffsetProperty(int32 aValue) throw (CannotSetValue);
	void refreshPreTriggerProperty(int32 aValue) throw (CannotSetValue);
	void refreshPulsePolarityProperty(int32 aValue) throw (CannotSetValue);
	void refreshUseFirstDerivativeProperty(int32 aValue) throw (CannotSetValue);
	void refreshCfdActiveProperty(int32 aValue) throw (CannotSetValue);
	void refreshCfdFractionProperty(int32 aValue) throw (CannotSetValue);
	void refreshCfdDelayProperty(int32 aValue) throw (CannotSetValue);
	void refreshCfdSignalZeroProperty(int32 aValue) throw (CannotSetValue);
	void refreshDynamicRangeProperty(int32 aValue) throw (CannotSetValue);

	virtual void updateBoardProperties();
	void updateNbBitsProperty();

	virtual void clearData();
	virtual void writeParam();
	virtual void initChannel(int32 aChannel);
	void initBoard();
};

}
#endif //CAENV1724CONTROLLER_H
