/*
 * Nomad Instrument Control Software
 *
 * Copyright 2011 Institut Laue-Langevin
 *
 * Licensed under the EUPL, Version 1.1 only (the "License");
 * You may not use this work except in compliance with the Licence.
 * You may obtain a copy of the Licence at:
 *
 * http://joinup.ec.europa.eu/software/page/eupl
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the Licence is distributed on an "AS IS" basis,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the Licence for the specific language governing permissions and
 * limitations under the Licence.
 */

#ifndef NSXTOOL_NSXTOOLCONTROLLER_H
#define NSXTOOL_NSXTOOLCONTROLLER_H

#include <Controller.h>
#include <cameo/cameo.h>

namespace nsxtool {

class NSXToolController : public ExperimentController,
						public controller::Stoppable {

public:
	//! Type of controller
	static const std::string TYPE;

	NSXToolController(const std::string& name);
	NSXToolController(const NSXToolController& controller);
	virtual ~NSXToolController();

	virtual void postConfiguration();

	virtual void start();
	virtual void stop();

	Property<std::string> serverEndpoint;
	Property<bool> initialized;
	Property<std::string> fileDirectory;
	Property<std::string> filename;
	ArrayProperty<float64> x;
	ArrayProperty<float64> y;
	ArrayProperty<float64> data;

private:
	bool initApplication();
	void readFile(const std::string& nexusFileName, std::string& fileContent);
	void subscriberLoop();
	void readProcessedData(const std::string& response);

	void resizeData(ArrayProperty<float64> & property, int size);

	static const std::string REMOTE_APPLICATION;
	static const std::string RESPONDER;
	static const std::string NSX_PUBLISHER;

	std::auto_ptr<cameo::Server> m_server;
	std::auto_ptr<cameo::application::Instance> m_remoteApplication;
	std::auto_ptr<cameo::application::Subscriber> m_subscriber;
	std::auto_ptr<boost::thread> m_subscriberThread;
};

}

#endif
