/*
 * Nomad Instrument Control Software
 *
 * Copyright 2011 Institut Laue-Langevin
 *
 * Licensed under the EUPL, Version 1.1 only (the "License");
 * You may not use this work except in compliance with the Licence.
 * You may obtain a copy of the Licence at:
 *
 * http://joinup.ec.europa.eu/software/page/eupl
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the Licence is distributed on an "AS IS" basis,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the Licence for the specific language governing permissions and
 * limitations under the Licence.
 */

#ifndef DIELECTRICS_REMOTEMATLABREQ_H
#define DIELECTRICS_REMOTEMATLABREQ_H

#include <Controller.h>
#include <cameo/cameo.h>
#include "drivers/utilities/null/NullDriver.h"

namespace dielectrics {

class Spectrums;

/*!
 * \brief Test command.
 */
class Test: private controller::Command {

public:
	Test(ExperimentController * c) :
		controller::Command(c, "test", &Test::test, this) {
	}

	virtual void test() = 0;
	void testCommand(bool logging = false) {
		command(logging, true);
	}
};

class RemoteMatlabReq : public ExperimentController,
									public controller::Start,
									public controller::Raz,
									public Test {

public:
	//! Type of controller
	static const std::string TYPE;

	RemoteMatlabReq(const std::string& name);
	RemoteMatlabReq(const RemoteMatlabReq& controller);
	virtual ~RemoteMatlabReq();

	virtual void postConfiguration();

	virtual void setEnabled(bool all, bool postconf = true);
	virtual void setDisabled(bool all = false);

	void addSpectrumsController(Spectrums * controller);

	virtual void start();
	virtual void raz();
	virtual void test();

	void stopRun();

	static const std::string RUN_SPECTRUMS_TEST;
	static const std::string RUN_SPECTRUMS;
	static const std::string RUN_SPECTRUMS_TIME;
	static const std::string RUN_SINGLE_FREQUENCY;

	DynamicProperty<std::string> requestTypeValues;
	Property<std::string> requestType;

	Property<double> fr;
	Property<double> frMin;
	Property<double> frMax;
	Property<int32> nMax;
	Property<double> time;
	Property<std::string> comment;

	Property<std::string> serverEndpoint;
	Property<bool> initialized;
	Property<int32> applicationState;

	ArrayProperty<int32> values;

	Property<int32> actualN;
	Property<double> actualTime;
	Property<int32> actualRun;

private:
	bool initApplication();
	void stopApplication();
	void subscriberLoop();

	static const std::string MATLAB_APPLICATION;
	static const std::string MATLAB_STOP_APPLICATION;
	static const std::string RESPONDER_NAME;
	static const std::string PUBLISHER_NAME;

	std::vector<Spectrums *> m_spectrumsControllers;

	std::auto_ptr<cameo::Server> m_server;
	std::auto_ptr<cameo::application::Instance> m_matlabApplication;
	std::auto_ptr<cameo::application::Requester> m_requester;
	std::auto_ptr<cameo::application::Subscriber> m_subscriber;
	std::auto_ptr<boost::thread> m_subscriberThread;

	DriverPtr<driver::NullDriver> m_driver;
};

}

#endif
