/*
 * Nomad Instrument Control Software
 *
 * Copyright 2011 Institut Laue-Langevin
 *
 * Licensed under the EUPL, Version 1.1 only (the "License");
 * You may not use this work except in compliance with the Licence.
 * You may obtain a copy of the Licence at:
 *
 * http://joinup.ec.europa.eu/software/page/eupl
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the Licence is distributed on an "AS IS" basis,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the Licence for the specific language governing permissions and
 * limitations under the Licence.
 */

#ifndef NPP_REMOTEDPPLIVEPROCESS_H
#define NPP_REMOTEDPPLIVEPROCESS_H

#include "../npp/DPPLiveProcess.h"
#include <cameo/cameo.h>
#include <lstdpp128/lstdpp.h>
#include <boost/property_tree/ptree.hpp>

namespace npp {

class RemoteDPPLiveProcess : public DPPLiveProcess {

public:
	//! Type of controller
	static const std::string TYPE;

	RemoteDPPLiveProcess(const std::string& name);
	RemoteDPPLiveProcess(const RemoteDPPLiveProcess& controller);
	virtual ~RemoteDPPLiveProcess();

	virtual void postConfiguration();

	virtual void stop();

protected:
	void initFileReading();
	void fileReadingLoop();
	std::auto_ptr<boost::thread> startFileReading();
	void finishFileReading(std::auto_ptr<boost::thread> thread);
	bool fileReadingMode();
	std::string getFileNameToRead();
	boost::property_tree::ptree writeListModeContextToPropertyTree(const lstdpp128::ListModeContext & listModeContext);

	Property<std::string> serverEndpoint;
	Property<int32> blockSize;
	Property<std::string> mode;
	Property<std::string> testFileName;
	Property<int32> refreshTimeS;
	Property<int32> sleepTimeMS;

	std::auto_ptr<cameo::Server> m_server;
	std::auto_ptr<cameo::application::Publisher> m_publisher;
	std::auto_ptr<lstdpp128::LiveReader> m_reader;

	boost::mutex m_mutex;

	static const std::string TEST_MODE;
	static const std::string FILE_MODE;
	static const std::string DIRECT_MODE;
};

}

#endif
