/*
 * Nomad Instrument Control Software
 *
 * Copyright 2011 Institut Laue-Langevin
 *
 * Licensed under the EUPL, Version 1.1 only (the "License");
 * You may not use this work except in compliance with the Licence.
 * You may obtain a copy of the Licence at:
 *
 * http://joinup.ec.europa.eu/software/page/eupl
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the Licence is distributed on an "AS IS" basis,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the Licence for the specific language governing permissions and
 * limitations under the Licence.
 */

#ifndef TAS_REMOTEMATLABPUB_H
#define TAS_REMOTEMATLABPUB_H

#include "RemoteMatlab.h"
#include "drivers/utilities/null/NullDriver.h"

namespace tas {

class RemoteMatlabPub : public RemoteMatlab {

public:
	//! Type of controller
	static const std::string TYPE;

	RemoteMatlabPub(const std::string& name);
	RemoteMatlabPub(const RemoteMatlabPub& controller);
	virtual ~RemoteMatlabPub();

	virtual void postConfiguration();

	void updateNumor();
	void updateCountStatusMessage();

	virtual void start();

private:
	bool initApplication();
	void sendNumor(int32 numor, bool live);
	void subscriberLoop();
	void sendImageToFluentd(const proto::NumorResponse& response);

	static const std::string MATLAB_APPLICATION;
	static const std::string NUMOR_PUBLISHER;
	static const std::string IMAGE_PUBLISHER;
	static const int32 LOG_TYPE;
	static const int32 LIVE_TYPE;

	std::auto_ptr<cameo::Server> m_server;
	std::auto_ptr<cameo::application::Instance> m_matlabApplication;
	std::auto_ptr<cameo::application::Publisher> m_publisher;
	std::auto_ptr<cameo::application::Subscriber> m_subscriber;
	std::auto_ptr<boost::thread> m_subscriberThread;

	DriverPtr<driver::NullDriver> m_driver;
};

}

#endif
