/*
 * Nomad Instrument Control Software
 *
 * Copyright 2011 Institut Laue-Langevin
 *
 * Licensed under the EUPL, Version 1.1 only (the "License");
 * You may not use this work except in compliance with the Licence.
 * You may obtain a copy of the Licence at:
 *
 * http://joinup.ec.europa.eu/software/page/eupl
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the Licence is distributed on an "AS IS" basis,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the Licence for the specific language governing permissions and
 * limitations under the Licence.
 */

#include "SimulatedQE65000Driver.h"


#include <DataProvider/Property/TypedProperty/ArrayProperty.h>
#include <DataProvider/Property/TypedProperty/Property.h>
#include <DeviceContainer/Def/CommonDef.h>
#include <DeviceContainer/DeviceDriverState.h>
#include <unistd.h>
#include <ctime>

#include <boost/random/normal_distribution.hpp>
#include <boost/math/distributions/normal.hpp>
#include <boost/random.hpp>
#include <boost/random/random_device.hpp>

#include "../../../../NomadServer/src/common/base/BaseTypes.h"

namespace oceanoptics {

const int32 SimulatedQE65000Driver::DATASIZE = 1044;

SimulatedQE65000Driver::SimulatedQE65000Driver(QE65000Driver* owner) :
		QE65000State(owner) {

	sigma = 1;

}

SimulatedQE65000Driver::~SimulatedQE65000Driver() {

}

void SimulatedQE65000Driver::init() {
	//Assume spectrum length is always the same (1044)
	owner()->size = DATASIZE;
}

void SimulatedQE65000Driver::start() {
	//Simulate integrationTime taken into account
	usleep(owner()->integrationTime());

	//delete
	delete[] owner()->xData();
	delete[] owner()->yData();

	float64* m_tempXData = new float64[owner()->size()];
	float64* m_tempYData = new float64[owner()->size()];

	// Use boost to have a Gaussian curve
	boost::random::mt19937 rng;
	boost::uniform_int<> fromOneToFiveHundred(1, 500);

	rng.seed(std::time(0));
	boost::variate_generator<boost::random::mt19937, boost::uniform_int<> > generator(rng, fromOneToFiveHundred);

	float32 mean = owner()->size() / 2;
	int32 deviation = generator();
	boost::math::normal nd (mean, deviation);

	for (int32 j = 0; j < owner()->size(); j++) {
		m_tempXData[j] = j;
		m_tempYData[j] = 10000 * pdf(nd, j); //j + 100;
	}

	owner()->xData.setSize(owner()->size());
	owner()->xData.set(m_tempXData);

	owner()->yData.set(m_tempYData);
	owner()->yData.setSize(owner()->size());

}

void SimulatedQE65000Driver::stop() {
	m_owner->sendProgressEvent(PROGRESSION_END_DEVICE_CONTAINER);
}

void SimulatedQE65000Driver::readStatus() {
	owner()->status.update(QE65000Driver::PLUGGED);
}

}

